define([
  "dojo/_base/declare",
  "dojo/on",
  "dijit/_WidgetBase",
  "dijit/_TemplatedMixin",
  "dijit/form/TimeTextBox",
  "dojo/text!./templates/DateTimeTextBox.html",
  "wagby/DateTextBox"
], function(declare, on, _WidgetBase, _TemplatedMixin, TimeTextBox, template, DateTextBox) {
  return declare("wagby.DateTimeTextBox", [_WidgetBase, _TemplatedMixin], {
    templateString: template,
    DEFAULT_TIME_VALUE: "00:00:00",
    DEFAULT_TIME_FORMAT: "HH:mm:ss",
    pickerMin: "T00:00:00",
    pickerMax: "T23:59:59",
    timeIncrement: "T00:15:00",
    timeSelectInputRE: /HM:(\d+)-(\d+):(\d+)/,

    postCreate: function() {
      this.inherited(arguments);
      this.initDateTextBox();
      this.initTimeTextBox();
    },

    _setIdAttr: function(id) {
      this.containerNode.setAttribute("id", id);
      this.valueNode.setAttribute("id", id + "_value");
    },

    _setNameAttr: function(name) {
      if(this.valueNode){
        this.valueNode.setAttribute("name", name);
      }
    },

    _setValueAttr: function(value) {
      var dateTimeValue = value;
      if (!dateTimeValue) {
        return;
      }
      if (dateTimeValue.indexOf(" ") === -1) {
        dateTimeValue = dateTimeValue + " " + this.DEFAULT_TIME_VALUE;
      }
      this.valueNode.value = value;
    },

    _getValueAttr: function() {
      if (!this.valueNode.value || this.valueNode.value === "") {
        return "";
      }
      if (this.valueNode.value.indexOf(" ") === -1) {
        this.valueNode.value = this.valueNode.value + " " + this.DEFAULT_TIME_VALUE;
      }
      return this.valueNode.value;
    },

    _setTimeSelectInputAttr: function(value) {
      if (!value) {
        return;
      }
      var props = this.timeSelectInputRE.exec(value);
      if (!props || props.length !== 4) {
        return;
      }
      this.pickerMin = this.getPickerTime(props[1]);
      this.pickerMax = this.getPickerTime(props[2]);
      this.timeIncrement = this.getTimeIncrementValue(props[3]);
    },

    getPickerTime: function(hour) {
      if (hour === "24") {
        return "T23:59:59";
      }
      return "T" + this.getPaddingValue(hour) + ":00:00";
    },

    getTimeIncrementValue: function(value) {
      var timeIncrementValue = value;
      if (timeIncrementValue === "0") {
        timeIncrementValue = "1";
      }
      return "T00:" + this.getPaddingValue(timeIncrementValue) + ":00";
    },

    getPaddingValue: function(value) {
      return ("0" + value).substr(-2);
    },

    initDateTextBox: function() {
      this.dateTextBox = new DateTextBox({
        'id': this.id + "_DateTextBox",
      }).placeAt(this.dateTextBox);
      if (!this.dateFormat || this.dateFormat === "") {
        this.dateFormat = this.dateTextBox.DEFAULT_DATE_FORMAT;
      }
      this.dateTextBox.set("dateFormat", this.dateFormat);
      this.dateTextBox.set("displayedValue",
                           _setDateWithFormat(this.valueNode.value, this.dateFormat));
      if (this.params.dayOffset) {
        this.dateTextBox.set("dayOffset", this.params.dayOffset);
      }
      this.dateTextBox.set("style", "width", this.dateTextBox.getWidthByDateFormat(this.dateFormat));
      var that = this;
      this.dateTextBox.on("change", function() {
        that.setFocusTimeTextBox();
      });
      on(this.dateTextBox.textbox, "blur", function() {
        that.autoFormatDate();
      });
      this.setCommonProperties(this.dateTextBox);
    },

    setFocusTimeTextBox: function() {
      var timeValue = this.timeTextBox.textbox.value;
      if (!timeValue || timeValue === "") {
        this.timeTextBox.focus();
      }
    },

    autoFormatDate: function() {
      var inputDateValue = this.dateTextBox.textbox.value;
      if (!inputDateValue || inputDateValue === "" || this.dateTextBox.isValid()) {
        return;
      }
      this.dateTextBox._setValueAttr(inputDateValue);
    },

    initTimeTextBox: function() {
      this.timeTextBox = new TimeTextBox({
        'id': this.id + "_TimeTextBox",
      }).placeAt(this.timeTextBox);
      if (!this.timeFormat || this.timeFormat === "") {
        this.timeFormat = this.DEFAULT_TIME_FORMAT;
      }
      this.timeTextBox.set("constraints", {
        timePattern: this.timeFormat,
        pickerMin: this.pickerMin,
        pickerMax: this.pickerMax,
        clickableIncrement: this.timeIncrement,
      });
      var dateTimeValue = this._getValueAttr();
      var timeValue = dateTimeValue.substring(dateTimeValue.indexOf(" ") + 1);
      this.timeTextBox.set("displayedValue", timeValue);
      this.timeTextBox.set("style", "width", this.getWidthFromTimeFormat(this.timeFormat));
      var that = this;
      this.timeTextBox.on("change", function() {
        that.complementTime();
      });
      this.setCommonProperties(this.timeTextBox);
    },

    complementTime: function() {
      var inputTimeValue = this.timeTextBox.textbox.value;
      var inputTimeLength = inputTimeValue.length;
      if (inputTimeLength === this.timeFormat.length) {
        return;
      }
      var complementTimeValue = convText2Date(_setDateWithFormat(this.dateTextBox.get("value"), this.dateTextBox.DEFAULT_DATE_FORMAT) + " " + inputTimeValue);
      this.timeTextBox.set("_onChangeActive", false);
      this.timeTextBox.set("value", "T" + _setDateWithFormat(complementTimeValue, this.DEFAULT_TIME_FORMAT));
      this.timeTextBox.set("_onChangeActive", true);
      this.timeTextBox.closeDropDown();
      this.setDateTimeValue();
    },

    setCommonProperties: function(widget) {
      var that = this;
      widget.on("change", function() {
        that.setDateTimeValue();
      });
      if (this.params.onKeyDown) {
        widget.on("keydown", function(event) {
          that.params.onKeyDown(event);
        });
      }
      if (this.params.onChange) {
        widget.on("change", function(event) {
          that.params.onChange(event);
        });
      }
      if (this.params.required) {
        widget.set("required", true);
      }
      if (this.params.tabIndex) {
        widget.set("tabIndex", this.params.tabIndex);
      }
    },

    setDateTimeValue: function() {
      var dateValue = this.dateTextBox.get("value");
      if (!dateValue || dateValue === "") {
        this.valueNode.value = "";
        this.timeTextBox.set("value", "");
        this.timeTextBox.set("displayedValue", "");
        return;
      }
      var timeValue = this.timeTextBox.get("value");
      if (!timeValue || timeValue === "") {
        timeValue = this.DEFAULT_TIME_VALUE;
      }
      this.valueNode.value = _setDateWithFormat(this.dateTextBox.get("value"), this.dateTextBox.DEFAULT_DATE_FORMAT)
        + " " + _setDateWithFormat(timeValue, this.DEFAULT_TIME_FORMAT);
    },

    getWidthFromTimeFormat: function(format) {
      var width = format.length;
      if (format.length <= 2) {
        width = 3;
      } else if (format.length >= 8) {
        width = this.dateTextBox.getWidthByTheme(format, width);
      }
      return width + "em";
    },

  });
});
