/**
 * モデルペインクラス
 */
define([
  "dojo/_base/declare",
  "dojo/dom",
  "dojo/dom-style",
  "dijit/registry",
  "wb/AclPaneContent",
  "wb/ActionPaneContent",
  "wb/InitialDataPaneContent",
  "wb/LayoutPaneContent",
  "wb/ModelOtherPaneContent",
  "wb/ModelitemPaneContent",
  "wb/Pane",
  "wb/SendmailPaneContent"
], function(
  declare,
  dom,
  domStyle,
  registry,
  AclPaneContent,
  ActionPaneContent,
  InitialDataPaneContent,
  LayoutPaneContent,
  ModelOtherPaneContent,
  ModelitemPaneContent,
  Pane,
  SendmailPaneContent
) {
  return declare(Pane, {
    constructor: function(initValue) {
      this.loadedTabContents = [];
      this.backToModelId = ""; // 参照先モデルから戻るモデルIDを保持する
    },

    setModel: function(model) {
      this.model = model;
    },

    initWidgets: function() {
    },

    createWidget: function(key) {
    },

    tabContentLoaded: function(title) {
      // 全タブのロード完了後に onContentsLoaded を実行する
      this.loadedTabContents.push(title);
      if (this.loadedTabContents.length === this.tabCount) {
        this.onContentsLoaded();
      }
    },

    getId: function() {
      return this.model.getId();
    },

    onPaneLoaded: function() {
      this.inherited(arguments);
      this.setContents();
      // 参照先からの戻りがある場合ボタンのクリックイベントをセットする
      this.setBtnBackToModelClickEvent();
    },

    onContentsLoaded: function() {
      // モデル項目一覧の入力欄初期化関数を実行する
      this.modelitemPaneContent.initWidgets();
    },

    setContents: function() {
      this.contents = [];
      this.tabContainer = registry.byId("modelDefinitionTabContainer");
      var that = this;
      // モデル項目
      this.modelitemPaneContent = new ModelitemPaneContent({
        parentPane: this
      });
      this.contents.push(this.modelitemPaneContent);
      this.modelitemPaneContent.contents.on("Show", function() {
        if (!that.model) {
          return;
        }
        that.modelitemPaneContent.show();
      });
      // 画面動作定義
      this.actionPaneContent = new ActionPaneContent({
        parentPane: this
      });
      // 画面タブ表示時にビューの項目設定の選択肢を更新する
      this.actionPaneContent.contents.on("Show", function() {
        that.actionPaneContent.show();
      });
      this.contents.push(this.actionPaneContent);

      // メール
      this.sendmailPaneContent = new SendmailPaneContent({
        parentPane: this
      });
      this.sendmailPaneContent.contents.on("Show", function() {
        that.sendmailPaneContent.show();
      });
      this.contents.push(this.sendmailPaneContent);

      // 権限設定
      this.aclPaneContent = new AclPaneContent({
        parentPane: this
      });
      this.aclPaneContent.contents.on("Show", function() {
        that.aclPaneContent.show();
      });
      this.contents.push(this.aclPaneContent);

      // レイアウト
      this.layoutPaneContent = new LayoutPaneContent({
        parentPane: this
      });
      // レイアウト名の選択肢を更新する
      this.layoutPaneContent.contents.on("Show", function() {
        that.layoutPaneContent.show();
      });
      this.layoutPaneContent.contents.on("Hide", function() {
        that.layoutPaneContent.hide();
      });
      this.contents.push(this.layoutPaneContent);

      // その他
      this.modelOtherPaneContent = new ModelOtherPaneContent({
        parentPane: this
      });
      this.modelOtherPaneContent.contents.on("Show", function() {
        that.modelOtherPaneContent.show();
      });
      this.contents.push(this.modelOtherPaneContent);

      // 初期データ
      this.initialDataPaneContent = new InitialDataPaneContent({
        parentPane: this
      });
      this.contents.push(this.initialDataPaneContent);
      // 各タブの内容をタブコンテナへ追加
      for (var i=0,l=this.contents.length; i<l; i++) {
        var tab = this.contents[i].contents;
        tab.on("Load", function() {
          that.tabContentLoaded(this.title);
        });
        this.tabContainer.addChild(tab);
      }
      this.tabCount = this.contents.length;
    },

    setEvent: function() {
    },

    showModelInfo: function() {
      if (!this.modelitemPaneContent.widgets || !this.modelitemPaneContent.widgets["model/@description"]) {
        return;
      }
      var widget = this.modelitemPaneContent.widgets["model/@description"].widget;
      widget.set("_onChangeActive", false);
      widget.set("value", this.model.repositoryItems["model/@description"].value);
      widget.set("readOnly", this.isLocked());
      if (!this.isLocked()) {
        widget.focus();
      }
      widget.set("_onChangeActive", true);
      widget = this.modelitemPaneContent.widgets["model/@id"].widget;
      widget.set("_onChangeActive", false);
      widget.set("value", this.model.repositoryItems["model/@id"].value);
      widget.set("readOnly", this.isLocked());
      widget.set("_onChangeActive", true);

      // サブモデルの場合、メインモデル名を表示する
      this.showMainModelInfo();
    },

    // メインモデル名を表示する
    showMainModelInfo: function() {
      this.mainModelId = this.model.action.repositoryItems["model/@mainmodelidref"].value;
      if (this.mainModelId) {
        domStyle.set("modelInfoTable", "width", "1020px");
        domStyle.set("mainModelInfoHeader", "display", "");
        dom.byId("mainModelId").innerHTML = this.mainModelId;
        btnGoToMainModel.set('title', this.mainModelId);
        domStyle.set("mainModelInfo", "display", "");
      } else {
        domStyle.set("modelInfoTable", "width", "660px");
        domStyle.set("mainModelInfoHeader", "display", "none");
        domStyle.set("mainModelInfo", "display", "none");
      }
    },

    show: function() {
      this.inherited(arguments);
      this.showModelInfo();
      this.modelitemPaneContent.setModel(this.model);

      // モデル設定画面表示時に選択状態のタブの初期処理を実行する
      if (this.modelitemPaneContent.contents.get("selected")) {
        // 「項目」
        this.modelitemPaneContent.show();
      } else if (this.actionPaneContent.contents.get("selected")) {
        // 「画面」
        this.actionPaneContent.show();
      } else if (this.sendmailPaneContent.contents.get("selected")) {
        // 「メール」
        this.sendmailPaneContent.show();
      } else if (this.aclPaneContent.contents.get("selected")) {
        // 「権限」
        this.aclPaneContent.show();
      } else if (this.layoutPaneContent.contents.get("selected")) {
        // 「レイアウト」
        this.layoutPaneContent.show();
      } else if (this.modelOtherPaneContent.contents.get("selected")) {
        // 「その他」
        this.modelOtherPaneContent.show();
      }
      // 選択肢モデルの場合、初期データタブを表示する
      this.setInitialDataPaneContent();
      var that = this;
      setTimeout(function() {
        // 「画面」等の設定エラーを取得する
        that.getModelError();
      }, 1);
      // 参照元からの遷移の場合、参照元モデルへ戻るボタンを表示する
      var display = 'none';
      if (this.backToModelId !== "") {
        display = 'table-cell';
      }
      domStyle.set('btnBackToReferenceModelContainer', 'display', display);
    },

    setBtnBackToModelClickEvent: function() {
      var that = this;
      btnBackToReferenceModel.on('click', function() {
        jfcModelitemStandby.show();
        setTimeout(function() {
          wagbyDesigner.showModelPane(that.backToModelId);
          that.backToModelId = "";
          domStyle.set('btnBackToReferenceModelContainer', 'display', 'none');
        }, 30);
      });
    },

    getRepositoryItemContainer: function(repositoryKey) {
      // 画面
      if (this.actionPaneContent.repositoryKeys[repositoryKey]) {
        return this.actionPaneContent;
      } else if (this.sendmailPaneContent.repositoryKeys[repositoryKey]) {
        // メール
        return this.sendmailPaneContent;
      }
      return null;
    },

    getModelError: function() {
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/" + that.model.getId() + "/error",
        options: {
          handleAs: "json"
        }
      }, function(data) {
        // モデルにエラーをセットする
        that.setModelErrorMessage(data);
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_MODEL_008; // "モデルのエラー情報の取得に失敗しました"
      });
    },

    setTabColorNormal: function() {
      var tabs = this.tabContainer.tablist.containerNode.childNodes;
      for (var i=0,l=tabs.length; i<l; i++) {
        domStyle.set(tabs[i], "background", "");
      }
    },

    setTabColorError: function(repositoryKey) {
      var errorTabIndex;
      if (actionRepositoryKeys[repositoryKey]) {
        errorTabIndex = 1;
      } else if (sendmailRepositoryKeys[repositoryKey]) {
        errorTabIndex = 2;
      }
      var errorTab = this.tabContainer.tablist.containerNode.childNodes[errorTabIndex];
      domStyle.set(errorTab, "background", "#ffff66");
    },

    setModelErrorMessage: function(data) {
      // モデルの各タブの色を通常に戻しておく
      this.setTabColorNormal();
      for (var i=0,l=data.length; i<l; i++) {
        var error = data[i];
        this.setTabColorError(error.key);
      }
    },

    getErrorModelitems: function() {
      var that = this;
      if (!this.model) {
        return;
      }
      wagbyDesigner.sendRequest({
        url: "/model/" + that.model.getId() + "/error/modelitem",
        options: {
          handleAs: "json"
        }
      }, function(data) {
        // モデル項目にエラーをセットする
        that.setErrorModelitem(data);
        if (Object.keys(data).length !== 0) {
          // エラーがある場合には、ユーザモデル一覧の「状態」欄を更新するため、
          // ユーザモデル一覧のデータを再取得する
//          wagbyDesigner.isNeedRefreshModelList = true;
        }
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_MODEL_009; // "モデル項目のエラー情報の取得に失敗しました"
      });
    },

    setErrorModelitem: function(data) {
      if (Object.keys(data).length === 0) {
        // モデル項目一覧の詳細ボタン、モデル項目詳細定義ダイアログのエラーをクリアする
        this.modelitemPaneContent.clearError();
        wagbyDesigner.modelitemDetailDialog.clearErrors();
        return;
      }
      for (var name in data) {
        if (name) {
          var modelitem = this.model.getModelitemByName(name);
          if (modelitem) {
            // モデル項目画面の「詳細」ボタンの色を変える
            this.modelitemPaneContent.setError(modelitem);
          }
        }
      }
      // モデル項目詳細定義ダイアログにエラーをセットする
      wagbyDesigner.modelitemDetailDialog.setErrors(data);
    },

    hide: function() {
      this.inherited(arguments);
      this.modelitemPaneContent.hide();
      this.actionPaneContent.hide();
      this.sendmailPaneContent.hide();
      this.aclPaneContent.hide();
      this.layoutPaneContent.hide();
      wagbyDesigner.releaseLock();
    },

    // 選択肢モデルかどうかに応じて初期データタブの表示・非表示を行なう
    setInitialDataPaneContent: function() {
      if (!this.model) {
        return;
      }
      // 初期データタブが既に追加されているか確認する
      var isInitialDataPaneContentExists = false;
      var tabContens = this.tabContainer.getChildren();
      for (var i=0,l=tabContens.length; i<l; i++) {
        if (tabContens[i].id === "modelPaneInitialDataContentFrame") {
          isInitialDataPaneContentExists = true;
          break;
        }
      }
      if (this.model.isCollectionModel()) {
        this.initialDataPaneContent.show();
        if (!isInitialDataPaneContentExists) {
          // 選択肢モデルで初期データタブが無い場合、追加する
          this.tabContainer.addChild(this.initialDataPaneContent.contents);
        }
      } else {
        // 通常モデルで初期データタブがタブ内に存在している場合、削除する
        if (isInitialDataPaneContentExists) {
          this.initialDataPaneContent.hide();
          this.tabContainer.removeChild(this.initialDataPaneContent.contents);
          this.tabContainer.selectChild(tabContens[0]);
        }
      }
    },

    destroy: function() {
      // タブ内のデータを削除
      for (var i=0,l=this.contents.length; i<l; i++) {
        if (this.contents[i] && this.contents[i].hide) {
          this.contents[i].hide();
        }
      }
    },

    setBackToModelId: function(modelId) {
      this.backToModelId = modelId;
      if (!modelId || modelId === "") {
        return;
      }
      var title = this.model.getDescriptionWithId();
      btnBackToReferenceModel.set('title', title);
    },

    refresh: function() {
    },

    isLocked: function() {
      if (!this.model) {
        return false;
      }
      return this.model.isLocked();
    },

    updateActionPaneRepositoryData: function(modifiedModelData) {
      for (repositoryKey in modifiedModelData) {
        if (this.model.action.repositoryItems[repositoryKey]) {
          this.actionPaneContent.updateRepositoryData(repositoryKey, modifiedModelData[repositoryKey]);
        }
      }
    },

    clearDataSetFlag: function() {
      this.modelitemPaneContent.isModelitemDataSet = false;
      this.actionPaneContent.isRepositoryDataSet = false;
      this.sendmailPaneContent.isRepositoryDataSet = false;
    },

  });
});
