/**
 * WagbyDesigner アプリケーションコントローラークラス
 */
define([
  "dojo/_base/array",
  "dojo/_base/declare",
  "dojo/_base/event",
  "dojo/_base/fx",
  "dojo/_base/kernel",
  "dojo/cookie",
  "dojo/dom",
  "dojo/dom-attr",
  "dojo/dom-style",
  "dojo/has",
  "dojo/i18n!../../wagbydesigner/nls/resource.js",
  "dojo/json",
  "dojo/on",
  "dojo/query",
  "dojo/request",
  "dojo/window",
  "dijit/ConfirmDialog",
  "dijit/Menu",
  "dijit/MenuItem",
  "dijit/Tooltip",
  "dijit/layout/BorderContainer",
  "dijit/layout/ContentPane",
  "dijit/registry",
  "wb/BrowseUserDefinedItemDialog",
  "wb/FunctionReferenceDialog",
  "wb/MenuPane",
  "wb/ModelListPane",
  "wb/ModelPane",
  "wb/ModelReferenceTypeDetailDialog",
  "wb/ModelReferenceTypeDetailRepository",
  "wb/ModelRepository",
  "wb/ModelitemBulkUpdateDialog",
  "wb/ModelitemDetailDialog",
  "wb/ModelitemDetailRepository",
  "wb/PaneManager",
  "wb/PrimitiveTypeDetailDialog",
  "wb/PrimitiveTypeDetailRepository",
  "wb/Project",
  "wb/ProjectEnvironmentPane",
  "wb/BuildPane",
  "wb/OperationsPane",
  "wb/ScreenResourceID",
  "wb/ScriptEditor",
  "wb/TotalviewMeasureDialog",
  "wb/UserDefinedItemDialog",
  "wb/UserDefinedItemSettingRepository",
  "wb/WidgetDefault"
], function(
  array,
  declare,
  baseEvent,
  fx,
  kernel,
  cookie,
  dom,
  domAttr,
  domStyle,
  has,
  jfcRes,
  JSON,
  on,
  query,
  request,
  win,
  ConfirmDialog,
  Menu,
  MenuItem,
  Tooltip,
  BorderContainer,
  ContentPane,
  registry,
  BrowseUserDefinedItemDialog,
  FunctionReferenceDialog,
  MenuPane,
  ModelListPane,
  ModelPane,
  ModelReferenceTypeDetailDialog,
  ModelReferenceTypeDetailRepository,
  ModelRepository,
  ModelitemBulkUpdateDialog,
  ModelitemDetailDialog,
  ModelitemDetailRepository,
  PaneManager,
  PrimitiveTypeDetailDialog,
  PrimitiveTypeDetailRepository,
  Project,
  ProjectEnvironmentPane,
  BuildPane,
  OperationsPane,
  ScreenResourceID,
  ScriptEditor,
  TotalviewMeasureDialog,
  UserDefinedItemDialog,
  UserDefinedItemSettingRepository,
  WidgetDefault
) {
  return declare(null, {
    constructor: function(releaseTimestamp) {
      // アプリケーション名
      this.APPNAME = "wagbydesigner";
      // URL
      this.BASEURL = "service";
      // テンプレートの HTML キャッシュ更新のためリリース日付を保持しておく
      this.RELEASE_TIMESTAMP = releaseTimestamp;
      // ビルド終了時等のサウンド出力設定のクッキー設定名
      this.SOUND_SETTING_COOKIE_NAME = "jfcWagbyDesignerSoundSetting";
      // レイアウト設定済みフラグ
      this.layoutSettingChanged = false;
      this.jfcRes = jfcRes;
      this.locale = kernel.locale;
      this.FA_URL = "https://fontawesome.com/icons?d=gallery&m=free";
      this.SCRIPT_DEFINED_MARK = "* ";
    },

    initScreen: function() {
      // レイアウト枠設定
      this.borderContainer = new BorderContainer({
      }, "windowBorderContainer");
      // ヘッダ領域設定
      var headerContent = new ContentPane({
        region: "top"
      }, "toparea");
      var that = this;
      headerContent.on("Show", function() {
        that.setWagbyDesignerMenu();
      });
      this.borderContainer.addChild(headerContent);

      // コンテンツ領域設定
      this.borderContainer.addChild(new ContentPane({
        region: "center",
        style: "margin: 0; padding: 10px;"
      }, "centerarea"));
      // 各コンテンツ設定(モデル一覧、モデル項目、プロジェクト設定)
      this.initPane();
      // レイアウト枠初期化処理(内部のウィジェット等の表示が初期化される)
      this.borderContainer.startup();
    },

    fadeInLoader: function() {
      domStyle.set("loader", "opacity", "0.9");
      domStyle.set("loader", "display", "");
    },

    fadeOutLoader: function() {
      fx.fadeOut({
        node: "loader",
        duration: 800,
        onEnd: function() {
          domStyle.set("loader", "display", "none");
        }
      }).play();
    },

    setWagbyDesignerMenu: function() {
      // WagbyDesigner メニューボタンのラベル設定
      this.setMenuLabel();
      // WagbyDesigner メニューボタンのツールチップ設定
      this.setTooltip();
    },

    setMenuLabel: function() {
      domAttr.set("showModelPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Model);
      domAttr.set("showMenuPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Menu);
      domAttr.set("showEnvironmentPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_ProjectEnvironment);
      domAttr.set("showBuildPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Build);
      domAttr.set("showOperationsPaneButton", "innerHTML", this.jfcRes.LABEL_MENU_Operation);
      domAttr.set("logoffButton", "innerHTML", this.jfcRes.LABEL_MENU_Logoff);
    },

    setTooltip: function() {
      new Tooltip({
        connectId: ["menuButtonModel"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Model, //"モデルの定義を行ないます"
        position: ["below"],
      });
      this.menuTooltip = new Tooltip({
        connectId: ["menuButtonMenu"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Menu, //"メニューの定義を行ないます"
        position: ["below"],
      });
      new Tooltip({
        connectId: ["menuButtonProjectEnvironment"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_ProjectEnvironment, //"環境設定を行ないます"
        position: ["below"],
      });
      new Tooltip({
        connectId: ["menuButtonBuild"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Build, //"アプリケーションのビルドを行ないます"
        position: ["below"],
      });
      new Tooltip({
        connectId: ["menuButtonOperations"],
        showDelay: 10,
        hideDelay: 10,
        label: this.jfcRes.LABEL_MENU_TOOLTIP_Operation, //"定義情報のバックアップ・リストアを行ないます"
        position: ["below"],
      });
    },

    initPane: function() {
      // ペイン表示・非表示を管理するマネージャオブジェクト
      this.paneManager = new PaneManager();
      // モデル一覧
      this.modelListPane = new ModelListPane({
        name: "modelList",
        href: "wb/paneTemplate/model/modelList.html?" + wagbyDesigner.RELEASE_TIMESTAMP
      });
      this.paneManager.addPane(this.modelListPane);

      var modelRepository = new ModelRepository();
      // モデル
      this.modelPane = new ModelPane({
        name: "model",
        repositoryKeys: modelRepository.modelRepositoryKeys,
        href: "wb/paneTemplate/model/model.html?" + wagbyDesigner.RELEASE_TIMESTAMP
      });
      this.paneManager.addPane(this.modelPane);

      // メニュー
      this.menuPane = new MenuPane({
        name: "menu"
      });
      this.paneManager.addPane(this.menuPane);

      // 環境
      this.projectEnvironmentPane = new ProjectEnvironmentPane({
        name: "projectEnvironment"
      });
      this.paneManager.addPane(this.projectEnvironmentPane);

      // ビルド
      this.buildPane = new BuildPane({
        name: "build",
        href: "wb/paneTemplate/build/build.html?" + wagbyDesigner.RELEASE_TIMESTAMP
      });
      this.paneManager.addPane(this.buildPane);

      // 運用
      this.operationsPane = new OperationsPane({
        name: "operations",
        href: "wb/paneTemplate/operations/operationPane.html?" + wagbyDesigner.RELEASE_TIMESTAMP
      });
      this.paneManager.addPane(this.operationsPane);
      // メニューボタンイベント
      this.setMenuButtonEvent();
    },

    init: function() {
      this.screenResourceID = new ScreenResourceID();
      // プロジェクト
      this.project = new Project();
      // ウィジェットデフォルト
      this.widgetDefault = new WidgetDefault(jfcRes, this.locale);
      // 画面設定
      this.initScreen();
      // ダイアログの初期化
      this.initDialog(jfcRes);
      this.initModelitemTypeMap();
      this.initMasterModelMap();
      // ブラウザウィンドウ変更時のイベントをセットする
      this.setWindowResizeEvent();
      // ビルド処理後の音設定ボタンの初期化
      this.setBtnSoundSetting();
      // 関連図のモデルデータ再生成フラグ
      this.relationModelDataNeedsRefresh = true;
      // ブラウザ判別
      this.setBrowserType();
    },

    setWindowResizeEvent: function() {
      on(window, "resize", this.getWindowResizeFunc(this.borderContainer));
    },

    getWindowResizeFunc: function(borderContainer) {
      var that = this;
      var timer = false;
      return function() {
        if (timer) {
          clearTimeout(timer);
        }
        timer = setTimeout(function() {
          that.paneManager.needsLayoutPane = true;
        }, 500);
      };
    },

    setMenuButtonEvent: function() {
    },

    initDialog: function() {
      this.browseUserDefinedItemDialog = new BrowseUserDefinedItemDialog({
        name: "browseUserDefinedItem",
        title: jfcRes.TITLE_UserDefinedItemSetting_DIALOG, // ユーザ定義項目設定
        size: {width: 900, height: 620},
        displayCancelButton: true,
        jfcRes: jfcRes,
      });

      this.functionReferenceDialog = new FunctionReferenceDialog({
        name: "functionReference",
        title: jfcRes.TITLE_FUNCTION_REFERENCE_DIALOG, // 関数入力
        size: {width: 600, height: 390},
        displayCancelButton: true,
        jfcRes: jfcRes,
      });

      this.modelitemBulkUpdateDialog = new ModelitemBulkUpdateDialog({
        name: "modelitemBulkUpdate",
        title: jfcRes.TITLE_MODELITEM_BULKUPDATE_DIALOG, // モデル項目詳細一括設定
        size: {width: 900, height: 600},
        displayCancelButton: true,
        jfcRes: jfcRes,
      });

      var modelitemDetailRepository = new ModelitemDetailRepository();
      // 項目詳細定義ダイアログ
      this.modelitemDetailDialog = new ModelitemDetailDialog({
        name: "modelitemDetail",
        title: jfcRes.TITLE_MODELITEM_DETAIL_DIALOG, // "モデル項目詳細定義",
        size: {width: 770, height: 600},
        repositoryKeys: modelitemDetailRepository.modelitemDetailRepositoryKeys,
        jfcRes: jfcRes,
      });

      // 項目の型ダイアログ(他のストアモデルの参照)
      var modelReferenceTypeDetailRepository = new ModelReferenceTypeDetailRepository();
      this.modelReferenceTypeDetailDialog = new ModelReferenceTypeDetailDialog({
        name: "modelReferenceTypeDetail",
        title: jfcRes.TITLE_TYPE_DETAIL_DIALOG, //"モデル項目の型詳細設定",
        size: {width: 550, height: 355},
        repositoryKeys: modelReferenceTypeDetailRepository.modelReferenceTypeDetailRepositoryKeys,
        displayCancelButton: true,
      });

      // 項目の型ダイアログ(文字列、数値、日付・時刻)
      var primitiveTypeDetailRepository = new PrimitiveTypeDetailRepository();
      this.primitiveTypeDetailDialog = new PrimitiveTypeDetailDialog({
        name: "primitiveTypeDetail",
        title: jfcRes.TITLE_TYPE_DETAIL_DIALOG, //"モデル項目の型詳細設定",
        size: {width: 500, height: 280},
        repositoryKeys: primitiveTypeDetailRepository.primitiveTypeDetailRepositoryKeys,
        displayCancelButton: true,
        jfcRes: jfcRes,
      });

      // 集計ビューメジャー定義ダイアログ
      this.totalviewMeasureDialog = new TotalviewMeasureDialog({
        name: "totalviewMeasureDialog",
        title: jfcRes.TITLE_ACTION_TOTALVIEW_MEASURE_DIALOG, // "メジャー項目定義"
        size: {width: 760, height: 600},
        repositoryKeys: modelitemDetailRepository.modelitemDetailRepositoryKeys,
      });
      this.totalviewMeasureDialog.dialog.set("style", "width: 760; height: 600; overflow-y: scroll;");

      var userDefinedItemSettingRepository = new UserDefinedItemSettingRepository();
      this.userDefinedItemDialog = new UserDefinedItemDialog({
        name: "userDefinedItem",
        title: jfcRes.TITLE_UserDefinedItemSetting_DIALOG, // ユーザ定義項目設定
        size: {width: 900, height: 620},
        repositoryKeys: userDefinedItemSettingRepository.userDefinedItemSettingRepositoryKeys,
        jfcRes: jfcRes,
      });
    },

    getModelitemDetailCategoryOptions: function() {
      return [
        {label: "", value: " ", selected: true},
        {label: this.jfcRes.LABEL_modelitemDetailRepository_Basic, value: "basic"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Detail, value: "detail"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Check, value: "check"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Input, value: "input"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Output, value: "output"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Search, value: "search"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Default, value: "default"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Implicit, value: "implicit"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Help, value: "help"},
        {label: this.jfcRes.LABEL_TAB_ModelitemDetailDialog_Other, value: "other"}
      ];
    },

    refreshModelListPane: function() {
      // ユーザモデルデータの取得
      this.setUserModelListData();
    },

    showModelListPane: function() {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      // 戻り先のモデルIDをクリアする
      this.modelPane.setBackToModelId("");
      // ユーザモデルデータの取得
      this.setUserModelListData();
    },

    setModelEditButtonEnable: function(id) {
      this.modelListPane.userModelListPaneContent.setModelEditButtonEnable(id);
    },

    showModelPane: function(modelId) {
      var that = this;
      this.releaseLock(function() {
        that.sendRequest({
          'url': "service/model/" + modelId,
          'options': {
            'query': {
              'lock': true,
            },
            'method': "GET",
            'handleAs': "json",
          }
        }, function(data) {
          var model = that.project.getModel(modelId);
          if (!model) {
            return;
          }
          that.activeModel = model;
          that.modelPane.clearDataSetFlag();
          that.modelPane.setModel(model);
          model.setEditable(data.editable === "true");
          // モデルのリポジトリデータを更新する
          model.modifyRepositoryData(data);
          // モデル項目のリポジトリデータを更新する
          model.getModelitemData({
            callback: function() {
              wagbyDesigner.paneManager.show(wagbyDesigner.modelPane);
            },
          });
        });
      });
    },

    showProjectEnvironmentPane: function() {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      this.paneManager.show(this.projectEnvironmentPane);
    },

    showMenuPane: function() {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      this.menuPane.refreshTitlePane();
      this.paneManager.show(this.menuPane);
    },

    showBuildPane: function() {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      this.paneManager.show(this.buildPane);
    },

    showOperationsPane: function() {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      this.paneManager.show(this.operationsPane);
    },

    logoff: function(logoutUrl) {
      if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
        return;
      }
      if (this.menuPane.isMenuPaneDisplayed) {
        this.menuPane.sendMenu(function() {
          location.href = logoutUrl;
        });
      } else {
        location.href = logoutUrl;
      }
    },

    checkAllModelListRow: function() {
      this.changeRowsCheckState({
        id: "userModelListBody",
        name: "rowSelect",
        checked: true
      });
    },

    uncheckAllModelListRow: function() {
      this.changeRowsCheckState({
        id: "userModelListBody",
        name: "rowSelect",
        checked: false
      });
    },

    checkAllModelitemListRow: function() {
      this.changeRowsCheckState({
        id: "modelitemListBody",
        name: "rowSelect",
        checked: true
      });
    },

    checkAllLayoutListRow: function() {
      this.changeRowsCheckState({
        id: "groupTabListBody",
        name: "rowSelect",
        checked: true
      });
    },

    uncheckAllLayoutListRow: function() {
      this.changeRowsCheckState({
        id: "groupTabListBody",
        name: "rowSelect",
        checked: false
      });
    },

    changeRowsCheckState: function(params) {
      var target = "#" + params.id + " input[name*='" + params.name +"']";
      query(target).forEach(function(node){
        registry.byId(node.id).set("value", params.checked);
      });
    },

    checkAllRow: function() {
      this.changeRowSelectCheckState({checked: true});
    },

    uncheckAllRow: function() {
      this.changeRowSelectCheckState({checked: false});
    },

    changeRowSelectCheckState: function(params) {
      query("input[name*='rowSelect']").forEach(function(node){
        registry.byId(node.id).set("value", params.checked);
      });
    },

    // モデル新規作成
    addNewModel: function() {
      this.modelListPane.userModelListPaneContent.deselectModels();
      var that = this;
      this.project.createModel({
        callback: function(model) {
          that.showModelPane(model.getId());
        }
      });
    },

    // コレクションモデル新規作成
    addCollectionModel: function() {
      this.modelListPane.userModelListPaneContent.deselectModels();
      this.project.addCollectionModel();
    },

    // コレクションモデル(初期データ固定)新規作成
    addCollectionModelStatic: function() {
      this.modelListPane.userModelListPaneContent.deselectModels();
      this.project.addCollectionModelStatic();
    },

    // モデル削除
    deleteModel: function() {
      var deleteModelIds = this.modelListPane.userModelListPaneContent.getSelectedModelIds();
      if (deleteModelIds.length === 0) {
        return;
      }
      // 削除対象のモデルを参照している項目等がないか確認する
      var deleteModel;
      var modifyMapOtherModel;
      var deleteModelName = "";
      var referencedItemMessage = "";
      for (var i=0,l=deleteModelIds.length; i<l; i++) {
        deleteModel = this.project.getModel(deleteModelIds[i]);
        deleteModelName += "- " + deleteModel.getDescription() + "\n";
        modifyMapOtherModel = this.checkItemReferenced(null, {
          url: this.BASEURL + "/model/" + deleteModel.getId(),
          method: "delete"
        });
        referencedItemMessage += this.getReferencedItemMessage(modifyMapOtherModel);
      }
      var confirmMessage = "";
      if (referencedItemMessage !== "") {
        confirmMessage += this.jfcRes.MSG_CONFIRM_DELETE_MODEL_1; // "削除対象のモデルは下記のモデルで参照されています\n\n";
        confirmMessage += (referencedItemMessage + "\n");
      }
      confirmMessage += this.jfcRes.MSG_CONFIRM_DELETE_MODEL_2 + deleteModelName; // "以下のモデルを削除してもよろしいですか?\n\n"
      if (!confirm(confirmMessage)) {
        return;
      }
      wagbyDesigner.fadeInLoader();
      this.modelListPane.userModelListPaneContent.deselectModels();
      var that = this;
      setTimeout(function() {
        that.project.deleteModels(deleteModelIds, function() {
          // モデル一覧再表示
          that.showModelListPane();
        });
      }, 10);
    },

    getShowModelPaneFunction: function(modelId) {
      var wagbyDesigner = this;
      return function() {
        wagbyDesigner.showModelPane(modelId);
      };
    },

    sortModelitemByNo: function() {
      this.activeModel.sortModelitemByNo();
    },

    addPrincipal: function() {
      // 指定無しの場合、最終行に追加する
      this.activeModel.acl.addPrincipalAtSelectedIds(this.getSelectedRowIds());
      // プリンシパルリスト再表示
      this.modelPane.aclPaneContent.refresh();
      // プリンシパル日本語が未入力の項目にフォーカスをセット
      this.modelPane.aclPaneContent.setFocusEmptyContent();
    },

    copyPrincipal: function() {
      var copyTargetIds = this.getSelectedRowIds();
      for (var i=0,l=copyTargetIds.length; i<l; i++) {
        this.activeModel.acl.copyPrincipal(copyTargetIds[i]);
      }
      // プリンシパルリスト再表示
      this.modelPane.aclPaneContent.refresh();
    },

    deletePrincipal: function() {
      var principalCount = this.activeModel.acl.getPrincipalCount();
      if (principalCount === 0) {
        return;
      }
      var selectedIds = this.getSelectedRowIds();
      if (selectedIds.length === 0) {
        // 削除対象行が選択されていない場合、最終行を削除対象とする
        var deleteTarget = this.activeModel.acl.principals[principalCount - 1];
        selectedIds.push(parseInt(deleteTarget.id));
      }
      // 削除確認メッセージを表示する
      if (!this.confirmDeletePrincipal(selectedIds)) {
        return;
      }
      var that = this;
      this.activeModel.acl.deletePrincipal(selectedIds, function() {
        // プリンシパルリスト再表示
        that.modelPane.aclPaneContent.show();
      });
    },

    addPermission: function() {
      this.activeModel.acl.addPermission();
      // プリンシパルリスト再表示
      this.modelPane.aclPaneContent.refresh();
      this.modelPane.aclPaneContent.setFocusAddedPermission();
    },

    getSelectedPermissionIds: function() {
      var selectedNodes = query("#permissionListHead tr th[class='selected'] > div > div:first-child");
      if (selectedNodes.length === 0) {
        return [];
      }
      var selectedId = [];
      array.forEach(selectedNodes, function(selectedNode) {
        var id = selectedNode.id.substring(selectedNode.id.lastIndexOf("_") + 1);
        if (id) {
          selectedId.push(parseInt(id));
        }
      });
      return selectedId;
    },

    deletePermission: function() {
      var principalCount = this.activeModel.acl.getPrincipalCount();
      if (principalCount === 0) {
        return;
      }
      var selectedIds = this.getSelectedPermissionIds();
      if (selectedIds.length === 0) {
        // 削除対象列が選択されていない場合、最終列を削除対象とする
        // 1行目のプリンシパルから削除対象のパーミッションを取得する
        var principal = this.activeModel.acl.principals[0];
        var permission = principal.permissions[principal.permissions.length - 1];
        // ユーザ定義のパーミッションでない場合、削除しない
        if (!permission.userDefinedPermission) {
          return;
        }
        selectedIds.push(parseInt(permission.getId()));
      }
      // 削除確認メッセージを表示する
      if (!this.confirmDeletePermission(selectedIds)) {
        return;
      }
      this.activeModel.acl.deletePermission(selectedIds);
      // プリンシパルリスト再表示
      this.modelPane.aclPaneContent.refresh();
    },

    getSelectedRowIds: function() {
      var selectedRowIds = [];
      query(".rowSelect input:checked").forEach(function(node) {
        selectedRowIds.push(parseInt(registry.byId(node.id).get("value")));
      });
      return selectedRowIds;
    },

    confirmDeletePrincipal: function(selectedIds) {
      var deleteLabels = "";
      for (var i=0,l=selectedIds.length; i<l; i++) {
        var deleteTarget = this.activeModel.acl.getPrincipalById(selectedIds[i]);
        var id = deleteTarget.getPrincipalId();
        var content = deleteTarget.getContent();
        // プリンシパル(日本語)、プリンシパル(英語)が未入力の場合、削除確認メッセージは作成しない
        if (!id && !content) {
          continue;
        }
        deleteLabels += " " + content + "(" + id + ")\n";
      }
      // 削除確認メッセージが作成されていない場合、確認無しで削除する
      if (deleteLabels === "") {
        return true;
      }
      var confirmMsg = this.jfcRes.MSG_CONFIRM_DELETE_PRINCIPAL + deleteLabels; // "以下のプリンシパルを削除してもよろしいですか?\n\n"
      return confirm(confirmMsg);
    },

    confirmDeletePermission: function(selectedIds) {
      var deleteLabels = "";
      for (var i=0,l=selectedIds.length; i<l; i++) {
        var permissions = this.activeModel.acl.functionAcls[0].permissions;
        var deleteTarget;
        for (var j=permissions.length - 1; j>=0; j--) {
          if (selectedIds[i] === permissions[j].getId()) {
            deleteTarget = permissions[j];
            break;
          }
        }
        var action = deleteTarget.getAction();
        var content = deleteTarget.getContent();
        // プリンシパル(日本語)、プリンシパル(英語)が未入力の場合、削除確認メッセージは作成しない
        if (!action && !content) {
          continue;
        }
        deleteLabels += " " + content + "(" + action + ")\n";
      }
      // 削除確認メッセージが作成されていない場合、確認無しで削除する
      if (deleteLabels === "") {
        return true;
      }
      var confirmMsg = this.jfcRes.MSG_CONFIRM_DELETE_PERMISSION  + deleteLabels; // "以下のパーミッションを削除してもよろしいですか?\n\n"
      return confirm(confirmMsg);
    },

    checkAllHasCondtion: function(checkbox) {
      var checked = false;
      if (checkbox.get("value")) {
        checked = true;
      }
      query("#modelitemListBody input[name*='hasConditionModel']").forEach(function(node){
        registry.byId(node.id).set("value", checked);
      });
    },

    checkAllHasList: function(checkbox) {
      var checked = false;
      if (checkbox.get("value")) {
        checked = true;
      }
      query("#modelitemListBody input[name*='hasListModel']").forEach(function(node){
        registry.byId(node.id).set("value", checked);
      });
    },

    showModelitemTypeDetailDialog: function(modelitem) {
      // ダイアログを閉じた時にツールチップが表示されないように
      // 型のリストボックスにフォーカスを当てる
      if (modelitem.widgets["model/modelitem/@typeSimple"].widget) {
        modelitem.widgets["model/modelitem/@typeSimple"].widget.focus();
      }
      if (modelitem.getTypeSimple() === "referModel") {
        this.modelReferenceTypeDetailDialog.show(modelitem);
      } else {
        this.primitiveTypeDetailDialog.show(modelitem);
      }
    },

    isModelitemTypeContainer: function(typeSimple) {
      if (typeSimple === "container" || typeSimple === "containerId") {
        return true;
      }
      return false;
    },

    isIdrefMasterModel: function(idref) {
      if (this.masterModelMap[idref]) {
        return true;
      }
      return false;
    },

    showModelitemDetailDialog: function(modelitemId) {
      this.modelitemDetailDialog.show(
        this.activeModel.getModelitemById(modelitemId).detail);
    },

    stopClickEvent: function(widgetItems) {
      array.forEach(widgetItems, function(widgetItem) {
        if (dom.byId(widgetItem.widgetPlaceId)) {
          widgetItem.widget.on("click", function(event) {
            baseEvent.stop(event);
          });
        }
      });
    },

    getModelitemTemplateModels: function() {
      var modelitemTemplateModels = [];
      for (var i=0,l=this.project.modelitemTemplateModels.length; i<l; i++) {
        var model = this.project.modelitemTemplateModels[i];
        if (model.getModelitemCount() === 0) {
          continue;
        }
        modelitemTemplateModels.push(model);
      }
      return modelitemTemplateModels;
    },

    getModelitemTemplateMenuItems: function() {
      var menuItems = [];
      var that = this;
      var modelitemTemplateModels = this.getModelitemTemplateModels();
      array.forEach(modelitemTemplateModels, function(model) {
        menuItems.push(new MenuItem({
          label: model.getDescription(),
          iconClass: "dijitIconNewTask",
          onClick: function() {
            that.addTemplateModelitems(model.getId());
          }
        }));
      });
      return menuItems;
    },

    setModelitemTemplateMenu: function() {
      var that = this;
      var jfcPopupMenuModelitemTemplate = registry.byId('jfcPopupMenuModelitemTemplate');
      var dijitMenu = new Menu();
      var menuItems = that.getModelitemTemplateMenuItems();
      array.forEach(menuItems, function(menuItem) {
        dijitMenu.addChild(menuItem);
      });
      jfcPopupMenuModelitemTemplate.destroyDescendants();
      jfcPopupMenuModelitemTemplate.set("popup", dijitMenu);
    },

    showModelitemBulkUpdateDialog: function() {
      this.modelitemBulkUpdateDialog.show();
    },

    showFunctionReferenceDialog: function(widget) {
      if (!widget) {
        return;
      }
      this.functionReferenceDialog.show(this.activeModel.getId(), widget);
    },

    // テンプレートモデル項目追加
    addTemplateModelitems: function(templateModelId) {
      // モデル項目一覧のローダーを表示する
      jfcModelitemStandby.show();
      var no;
      // テンプレートモデル項目を追加する行を取得する
      var selectedRowIds = this.getSelectedRowIds();
      if (selectedRowIds.length > 0) {
        var selectedModelitem = this.activeModel.getModelitemById(parseInt(selectedRowIds[0]));
        no = selectedModelitem.getNo();
      }
      this.activeModel.addTemplateModelitems(no, templateModelId, function() {
        // モデル項目一覧のローダーを非表示にする
        jfcModelitemStandby.hide();
      });
    },

    showDialog: function (message, option) {
      var that = this;
      var html = "";
      if (message) {
        html = message.replace(/\n/g, "<br>");
      }
      var dialog = registry.byId("messageDialog");
      if (dialog) {
        var content = "<div style='margin: 10px;'>"
        + "<div style='margin: 20px 0;'>" + html + "</div>"
        + "<div style='text-align: center;'>"
        + "<button data-dojo-type=\"dijit/form/Button\" onclick=\"messageDialog.hide();\">" + that.jfcRes.LABEL_BUTTON_CLOSE + "</button>" // 閉じる
        + "</div></div>";
        dialog.set("content", content);
        dialog.show();
        dialog.on("hide", function() {
          if (option && option.isReturnLogon) {
            location.href = "/wagbydesigner/logon.jsp";
          }
        });
      }
    },

    showFatalErrorDialog: function (message) {
      this.showDialog(message, {isReturnLogon: true});
    },

    showTotalviewMeasureDialog: function(no) {
      this.totalviewMeasureDialog.show(no);
    },

    // モデルIDのリポジトリデータを更新する
    modifyOtherModelModelData: function(modelId, modifyMapOtherModel) {
      var model = this.project.getModel(modelId);
      if (!model) {
        return;
      }
      if (model.modifyRepositoryData && modifyMapOtherModel.modelMap) {
        model.modifyRepositoryData(modifyMapOtherModel.modelMap);
        this.modelPane.updateActionPaneRepositoryData(modifyMapOtherModel.modelMap);
      }
      if (modifyMapOtherModel.modelitemMap) {
        this.modifyOtherModelModelitemData(model, modifyMapOtherModel.modelitemMap);
      }
    },

    // モデルIDのリポジトリデータを更新する
    modifyOtherModelModelitemData: function(model, modelitemMap) {
      for (modelitemName in modelitemMap) {
        var modelitem = model.getModelitemByName(modelitemName);
        if (modelitem) {
          modelitem.modifyRepositoryData(modelitemMap[modelitemName]);
        }
      }
    },

    // モデルID、モデル項目IDを変更時に参照されているリポジトリを修正した
    // データがサーバから送信されるので、モデル毎にリポジトリの更新を行なう
    modifyOtherModelData: function(modifyMapOtherModel) {
      for (modelId in modifyMapOtherModel) {
        if (modifyMapOtherModel[modelId].modelMap) {
          this.modifyOtherModelModelData(modelId, modifyMapOtherModel[modelId]);
        }
      }
    },

    // 設定変更に関連する自モデルの設定値変更を行なう
    modifyThisModelData: function(modelId, repositoryKey, modifyMap) {
      if (Object.keys(modifyMap).length === 0) {
        return;
      }
      var modelMap = {};
      for (var key in modifyMap) {
        if (key === repositoryKey) {
          continue;
        }
        modelMap[key] = modifyMap[key];
      }
      var modifyMapThisModel = {};
      modifyMapThisModel.modelMap = modelMap;
      this.modifyOtherModelModelData(modelId, modifyMapThisModel);
    },

    // 設定変更に関連する自モデルの設定値変更を行なう
    modifyThisModelitemData: function(modelId, modifyMap) {
      if (Object.keys(modifyMap).length === 0) {
        return;
      }
      this.modifyOtherModelModelData(modelId, {
        modelitemMap: modifyMap,
      });
    },

    updateUserDefinedItemSetting: function(userDefinedItemSetting) {
      this.modelListPane.userDefinedItemSettingListPaneContent.updateUserDefinedItemSetting(userDefinedItemSetting);
    },

    initModelitemTypeMap: function() {
      this.typeDetailToTypeSimple = {
        "string": "string", // 文字列
        "zipcode": "string", // 郵便番号
        "mailaddress": "string", // メールアドレス
        "URL": "string",
        "filename": "string", // ファイル名
        "number": "number", // 整数
        "byte": "number", // 1バイト整数
        "short": "number", // 2バイト整数
        "integer": "number", // 4バイト整数
        "long": "number", // 8バイト整数
        "float": "number", // 4バイト浮動小数点
        "double": "number", // 8バイト浮動小数点
        "date": "dateTime", // 日付
        "dateTime": "dateTime", // 日付と時間
        "time": "dateTime", // 時間
        "referenceMaster_listbox": "referModel", // マスタモデルの参照(コンボボックス)
        "referenceMaster_radiobutton": "referModel", // マスタモデルの参照(ラジオボタン)
        "referenceMaster_checkbox": "referModel", // マスタモデルの参照(チェックボックス)
        "reference_listbox": "referModel", // 他のストアモデルの参照(コンボボックス)
        "reference_radiobutton": "referModel", // 他のストアモデルの参照(ラジオボタン)
        "reference_checkbox": "referModel", // 他のストアモデルの参照(チェックボックス)
        "reference_searchWindow": "referModel", // 他のストアモデルの参照(検索画面)
        "reference_lookup": "referModel", // 他のストアモデルの参照連動
        "container": "container", // (繰り返し項目のコンテナ)
        "containerId": "containerId", // (繰り返し項目コンテナ用のID)
        "createuserid": "string", // 固定値(作成したユーザのID)
        "updateuserid": "string", // 固定値(更新したユーザのID)
        "createdatetime": "dateTime", // 固定値(作成した日付と時間)
        "updatedatetime": "dateTime" // 固定値(更新した日付と時間)
      };

      this.typeSimpleDefaultDetailType = {
        "string": "string", // 文字列
        "number": "number", // 整数
        "dateTime": "date", // 日付
        "container": "container", // (繰り返し項目のコンテナ)
        "containerId": "containerId" // (繰り返し項目コンテナ用のID)
      };
    },

    initMasterModelMap: function() {
      this.masterModelMap = {
        "compulsoryChange_m": true, // juserのカスタマイズコードでマスタモデルとして扱っている。
        "jfcdispositiontype_m": true, // "ブラウザで表示", "ダウンロード"
        "jfcenable_m": true, // "有効"
        "jfclockforall_m": true, // "いいえ", "はい"
        "jfcmailtemplate_auth_m": true, // "利用しない", "利用する"
        "jfcmailtemplate_mailtype_m": true, // "テキストのみ", "書式付きテキストのみ", "ソースのみ", "テキストと書式付きテキスト", "テキストとHTMLソース", "EMLファイル"
        "jfcmailtemplate_protocol_m": true, // "SMTP", "SMTP over SSL"
        "jfcmailtemplate_popauthprotocol_m": true, // "POP3", "POP3 over SSL"
        "jfcmodelcreatetype_m": true, // "システム", "ユーザー"
        "jfcmodelkind_m": true, // "ストアモデル", "マスタモデル"
        "jfcmodeluseflag_m": true, // "有", "無"
        "jfcportletType_m": true, // ポートレットタイプ
        "jfcwfnodeType_m": true, // "順次", "合議"
        "jfcworkflow_mstype_m": true, // "新規登録", "申請", "承認", "却下", "保留", "差し戻し", "取り消し", "決裁"
        "jfcworkflow_remand_m": true, // "一つ前へ戻
        "jprincipal": true // プリンシパル
      };
    },

    capFirst: function(str) {
      if (!str) {
        return "";
      }
      var name = this.convClassname4Relax(str);
      return name.substring(0, 1).toUpperCase() + name.substring(1);
    },

    convClassname4Relax: function(s) {
      if (!s) {
        return "";
      }
      var convStr = "";
      for (var i=0,l=s.length; i<l; i++) {
        var c = s.charAt(i);
        if (c == '_') {
          while (i+1 < s.length && (c = s.charAt(++i)) == '_') ;
          if (c != '_') {
            convStr += c.toUpperCase();
          }
        } else {
          convStr += c;
        }
      }
      return convStr;
    },

    // リストボックスの選択肢を作成する
    refreshSelectOptions: function(args) {
      var selectWidget = args.widget;
      if (!selectWidget) {
        return;
      }
      selectWidget.removeOption(selectWidget.getOptions());
      selectWidget.addOption(args.options);
      if (args.defaultValue) {
        selectWidget.set("_onChangeActive", false);
        selectWidget.set("value", args.defaultValue);
        selectWidget.set("_onChangeActive", true);
      }
    },

    checkItemReferenced: function(widgetItem, overwriteOptions) {
      var modifyMapOtherModel;
      var options = {
        headers: {
          'Content-Type': "application/x-www-form-urlencoded; charset=utf-8",
          'x-wd-check-only': true
        },
        sync: true,
        method: "PUT",
        handleAs: "json"
      };
      var url;
      if (widgetItem) {
        url = widgetItem.getUrl();
        var data = {};
        data[widgetItem.getRepositoryKey()] = widgetItem.widget.get("value");
        options.data = data;
      }
      if (overwriteOptions) {
        if (overwriteOptions.url) {
          url = overwriteOptions.url;
        }
        if (overwriteOptions.method) {
          options.method = overwriteOptions.method;
        }
      }
      request(url, options).then(function(data) {
        if (data.updateModelInfo && data.updateModelInfo.modifyMapOtherModel) {
          modifyMapOtherModel = data.updateModelInfo.modifyMapOtherModel;
        } else if (data.updateModelitemInfo && data.updateModelitemInfo.modifyMapOtherModel) {
          modifyMapOtherModel = data.updateModelitemInfo.modifyMapOtherModel;
        }
      }, function(err) {
        if (widgetItem) {
          widgetItem.errorOnSendData(err.response.data, widgetItem, widgetItem.widget.get("value"));
        }
      });
      return modifyMapOtherModel;
    },

    getReferencedItemMessage: function (modifyMapOtherModel) {
      var message = "";
      for (var modelId in modifyMapOtherModel) {
        message += (wagbyDesigner.jfcRes.LABEL_MODEL_ID + ":" + modelId + " " + wagbyDesigner.jfcRes.LABEL_MODELITEM_ID + ":");
        var modelitemMap = modifyMapOtherModel[modelId].modelitemMap;
        for (var modelitemId in modelitemMap) {
          message += (modelitemId + " ");
        }
        message += "\n";
      }
      return message;
    },

    // 参照設定されているモデルID、項目ID変更時の確認ダイアログメッセージ
    getReferencedItemChangeConfirmMessage: function (modifyMapOtherModel) {
      var message = this.jfcRes.MSG_CONFIRM_MODIFY_MODEL_MODELITEM; // この項目は下記のモデルで参照されています\n変更してもよろしいですか？\n(参照設定されている値も全て変更されます)
      message += this.getReferencedItemMessage(modifyMapOtherModel);
      return message;
    },

    // 参照設定されているモデルID、項目ID変更時に確認を行なう
    confirmChangeReferencedItemValue: function(widgetItem, oldValue, modifyMapOtherModel) {
      var isConfirmed = true;
      if (Object.keys(modifyMapOtherModel).length > 0) {
        // 確認ダイアログを表示する
        if (!confirm(this.getReferencedItemChangeConfirmMessage(modifyMapOtherModel))) {
          // 変更しない場合、値を元に戻す
          isConfirmed = false;
          widgetItem.widget.set("_onChangeActive", false);
          widgetItem.widget.set("value", oldValue);
          widgetItem.widget.set("_onChangeActive", true);
        }
      }
      return isConfirmed;
    },

    createSubModel: function() {
      // コピー元となるメインモデルが一つ選択されているか確認する
      var mainModelIds = this.modelListPane.userModelListPaneContent.getSelectedModelIds();
      if (mainModelIds.length === 0 || mainModelIds.length > 1) {
        this.showDialog(this.jfcRes.MSG_ALERT_SELECT_MODEL_MAINMODEL); // "メインモデルとなるモデルを一つ選択して下さい"
        return;
      }
      this.modelListPane.userModelListPaneContent.deselectModels();
      this.project.createSubModel(mainModelIds[0]);
    },

    getForeignModels: function() {
      return this.project.getForeignModels();
    },

    getSubModels: function() {
      return this.project.getSubModels();
    },

    getReferencedModels: function() {
      return this.project.getReferencedModels();
    },

    getTypeLabel: function(type) {
      if (type === "string") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_1; // 文字列
      } else if (type === "zipcode") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_2; // 郵便番号
      } else if (type === "mailaddress") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_3; // メールアドレス
      } else if (type === "filename") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_string_5; // ファイル名
      } else if (type === "number") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_1; // 整数
      } else if (type === "byte") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_2; // 1バイト整数
      } else if (type === "short") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_3; // 2バイト整数
      } else if (type === "integer") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_4; // 4バイト整数
      } else if (type === "long") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_5; // 8バイト整数
      } else if (type === "float") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_6; // "4バイト浮動小数"
      } else if (type === "double") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_number_7; // "8バイト浮動小数"
      } else if (type === "date") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_dateTime_1; // "日付
      } else if (type === "dateTime") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_dateTime_2; // "日付と時刻"
      } else if (type === "time") {
        return this.jfcRes.OPTION_model_modelitem_typeDetail_dateTime_3; // "時刻"
      } else if (type.indexOf("listbox") !== -1) {
        return this.jfcRes.LABEL_INPUT_TYPE_SELECT; // "リストボックス"
      } else if (type.indexOf("radiobutton") !== -1) { // "ラジオボタン"
        return this.jfcRes.LABEL_INPUT_TYPE_RADIO; // "ラジオボタン"
      } else if (type.indexOf("checkbox") !== -1) { // "チェックボックス"
        return this.jfcRes.LABEL_INPUT_TYPE_CHECK; // "チェックボックス"
      } else if (type.indexOf("searchWindow") !== -1) { // "検索画面"
        return this.jfcRes.LABEL_INPUT_TYPE_SEARCH; // "検索画面"
      } else if (type.indexOf("lookup") !== -1) { // "参照連動"
        return this.jfcRes.LABEL_INPUT_TYPE_REF; // "参照連動"
      }
      return type;
    },

    sendRequest: function(properties, callback, errorHandler) {
      var that = this;
      var requestOptions = properties.options || {};
      if (!requestOptions.headers) {
        requestOptions.headers = {};
      }
      if (!requestOptions.headers['Content-Type']) {
        requestOptions.headers['Content-Type'] = "application/x-www-form-urlencoded; charset=utf-8";
      }
      // Dojo ライブラリで JSON データのパース処理を行なわないように
      // レスポンスデータの取り扱い方法についてのプロパティは変数へ
      // 保存して削除する
      var handleAs = requestOptions.handleAs;
      delete requestOptions['handleAs'];

      var url = properties.url;
      if (url.lastIndexOf(that.BASEURL, 0) === -1) {
        url = that.BASEURL + url;
      }
      request(url, requestOptions).then(function(data) {
        if (data) {
          if (data.response && data.response.status === 0) {
            wagbyDesigner.checkServerConnection(err);
          } else if (data !== "" && handleAs && handleAs.toLowerCase() === "json") {
            // レスポンスデータの取り扱いが JSON の場合には、パース処理を行なう
            if (data.indexOf("<html>") !== -1) {
              // JSON データとして取り扱うデータのレスポンスが HTML の場合はエラーとする
              that.showFatalErrorDialog(that.jfcRes.MSG_ALERT_DISCONNECT_SERVER); // "WagbyDesigner サーバとの接続が切断されました<br/> ログオン画面を表示します"
              return;
            }
            try {
              data = JSON.parse(data);
            } catch(e) {
              console.error("WagbyDesigner.sendRequest JSON.parse error: " + e);
            }
          }
        }
        if (callback) {
          callback(data);
        }
      }, function(err) {
console.error("sendRequest Error: ", err);
        var message;
        if (errorHandler) {
          message = errorHandler(err) || "";
        } else {
          message = that.jfcRes.MSG_ERROR_SEND_REQUEST; // "リポジトリデータ処理中にエラーが発生しました"
        }
        if (err && err.response.status === 0) {
          wagbyDesigner.checkServerConnection(err);
        } else if (err && err.response.status >= 500 && err.response.status < 600) {
          if (err.response && err.response.data) {
            message += err.response.data;
          } else {
            console.log(err);
          }
        }
        if (message && message !== "") {
          wagbyDesigner.showDialog(message);
        }
      });
    },

    enableAllWidgets: function(widgets) {
      this.setWidgetDisabled(widgets, false);
    },

    disableAllWidgets: function(widgets) {
      this.setWidgetDisabled(widgets, true);
    },

    setWidgetDisabled: function(widgets, disabled) {
      array.forEach(widgets, function(widget) {
        widget.set("disabled", disabled);
      });
    },

    checkServerConnection: function(data) {
      if (data && data.response.status === 0) {
        this.showFatalErrorDialog(this.jfcRes.MSG_ALERT_DISCONNECT_SERVER); // "WagbyDesigner サーバとの接続が切断されました<br/> ログオン画面を表示します"
      }
    },

    openScriptEditor: function(widget, mode) {
      if (!this.scriptEditor) {
        this.aceEditor = dom.byId("aceEditor");
        this.scriptEditor = new ScriptEditor();
      }
      // ブラウザ画面の70%の大きさをエディタサイズとする
      var width = Math.floor(win.getBox().w * 0.7);
      var height = Math.floor(win.getBox().h * 0.7);
      domStyle.set(this.aceEditor, "height", height + "px");
      domStyle.set(this.aceEditor, "width", width + "px");
      this.scriptEditor.setScriptWidget(widget);
      this.scriptEditor.setMode(mode);
      scriptEditorDialog.show();
    },

    getProjectId: function() {
      return this.projectEnvironmentPane.getProjectId();
    },

    getTomcatHttpPort: function() {
      return this.projectEnvironmentPane.getTomcatHttpPort();
    },

    refreshProjectEnvironmentRepositoryItems: function() {
      this.projectEnvironmentPane.projectEnvironment.refreshRepositoryItems();
    },

    setScreenResources: function(screenId) {
      var resourceIDs = wagbyDesigner.screenResourceID.getResourceIDs(screenId);
      if (!resourceIDs) {
        console.error("ScreenId:["+screenId+"] resource NOT FOUND");
        return;
      }
      var that = this;
      setTimeout(function() {
        for (var i=0,l=resourceIDs.length; i<l; i++) {
          var resourceID = resourceIDs[i];
          that.setScreenResource(resourceID.key, resourceID.id);
        }
      }, 1);
    },

    setScreenResource: function(key, id) {
      var that = this;
      if (!id) {
        id = key;
      }
      var node = dom.byId(id);
      if (node) {
        var defaultLabel = domAttr.get(node, "innerHTML");
//        if (defaultLabel !== that.jfcRes[key]) {
//          console.error("id:["+id+"] resource label different, defaultLabel:["+defaultLabel+"], res:["+that.jfcRes[key]+"]");
//        }
        domAttr.set(node, "innerHTML", that.jfcRes[key]);
      } else {
        console.error("node:["+id+"] NOT FOUND");
      }
    },

    setTabColorError: function(tabContainer, errorTabIndex) {
      if (errorTabIndex <= 0 ||
        errorTabIndex > tabContainer.tablist.containerNode.childNodes.length) {
        return;
      }
      var errorTab = tabContainer.tablist.containerNode.childNodes[errorTabIndex - 1];
      if (!errorTab) {
        return;
      }
      domStyle.set(errorTab, "background", "#ffff66");
    },

    setTabColorNormal: function(tabContainer) {
      var tabs = tabContainer.tablist.containerNode.childNodes;
      for (var i=0,l=tabs.length; i<l; i++) {
        domStyle.set(tabs[i], "background", "");
      }
    },

    createConfirmDialog: function(args) {
      var that = this;
      var confirmDialog;
      confirmDialog = new ConfirmDialog({
        title: args.title || "WagbyDesigner",
        style: args.style || "width: 350px",
        autofocus: false
      });
      domStyle.set(confirmDialog.actionBarNode, {
        'text-align': "right",
        'margin': "10px"
      });
      domStyle.set(confirmDialog.containerNode, 'whiteSpace', "pre");
      confirmDialog.set("content", args.message || "");
      confirmDialog.okButton.set("label", args.okButtonLabel || that.jfcRes.LABEL_BUTTON_YES); // はい
      confirmDialog.cancelButton.set("label", args.cancelButtonLabel || that.jfcRes.LABEL_BUTTON_NO); // いいえ

      confirmDialog.on("Execute", function() {
        confirmDialog.onExecuteFunc();
      });
      confirmDialog.on("Cancel", function() {
        confirmDialog.onCancelFunc();
      });
      confirmDialog.on("Show", function() {
        confirmDialog.onShowFunc();
      });
      confirmDialog.on("Hide", function() {
        confirmDialog.onHideFunc();
      });

      confirmDialog.setEventCallbackFunc = function(args) {
        confirmDialog.onExecuteFunc = args.onExecute || function(){};
        confirmDialog.onCancelFunc = args.onCancel || function(){};
        confirmDialog.onShowFunc = args.onShow || function(){};
        confirmDialog.onHideFunc = args.onHide || function(){};
      }
      confirmDialog.setEventCallbackFunc(args);
      return confirmDialog;
    },

    setBtnSoundSetting: function() {
      // クッキーから音設定の有無を取得してアイコン画像を切り替える
      var that = this;
      var soundSetting = cookie(that.SOUND_SETTING_COOKIE_NAME);
      var isSoundOn = false;
      if (soundSetting && soundSetting === "true") {
        isSoundOn = true;
      }
      if (isSoundOn) {
        btnSoundSetting.set("checked", isSoundOn);
        that.setBtnSoundSettingIcon({isSoundOn: isSoundOn});
        that.setSoundSettingCookie({isSoundOn: isSoundOn});
      }
      btnSoundSetting.on("click", this.setBtnSoundSettingClickEvent());
    },

    setBtnSoundSettingClickEvent: function() {
      var that = this;
      return function() {
        var isSoundOn = btnSoundSetting.get("checked");
        that.setBtnSoundSettingIcon({isSoundOn: isSoundOn});
        that.setSoundSettingCookie({isSoundOn: isSoundOn});
      };
    },

    setSoundSettingCookie: function(args) {
      cookie(this.SOUND_SETTING_COOKIE_NAME, args.isSoundOn, {expires: 365});
    },

    setBtnSoundSettingIcon: function(args) {
      var iconClass = "jfcSoundIcon jfcSoundOffIcon";
      if (args.isSoundOn) {
        iconClass = "jfcSoundIcon jfcSoundOnIcon";
      }
      btnSoundSetting.set("iconClass", iconClass);
    },

    isSoundOn: function() {
      return btnSoundSetting.get("checked");
    },

    isReferModelitemTypeZipcode: function(idref, rolecontent) {
      var model = this.project.getModelById(idref);
      if (!model) {
        return false;
      }
      // 参照先項目が参照連動の場合、参照先のモデルを確認する
      var modelitem = model.getModelitemByName(rolecontent);
      if (!modelitem) {
        return false;
      }
      if (modelitem.isReferModel()) {
        return this.isReferModelitemTypeZipcode(
          modelitem.getIdref(),
          modelitem.getRolecontent()
        );
      }
      var type = modelitem.getType();
      var addressfield = modelitem.getAddressfield();
      if (type === "zipcode" &&
          addressfield && (addressfield !== "" || addressfield !== " ")) {
        return true;
      }
      return false;
    },

    isChecked: function(value) {
      if (value === wagbyDesigner.jfcRes.OPTION_CHECKED) {
        return true;
      }
      return false;
    },

    bubbleInfoMessage: function(message) {
      this.bubbleMessage(message, "info");
    },

    bubbleErrorMessage: function(message) {
      this.bubbleMessage(message, "error");
    },

    bubbleMessage: function(message, type) {
      if (!message || message === "") {
        return;
      }
      jfcMessageToaster.setContent(message, type);
      jfcMessageToaster.show();
    },

    setRelationModelDataNeedsRefresh: function(flag) {
      this.relationModelDataNeedsRefresh = flag;
    },

    getRelationModelDataNeedsRefresh: function() {
      return this.relationModelDataNeedsRefresh;
    },

    // テキスト入力欄にフォーカスをセットして入力文字列を選択状態にする
    focusAndSelect: function(textbox) {
  	if (!textbox) {
        return;
      }
      textbox.focus();
      setTimeout(function() {
        var length = 0;
        if (textbox.value) {
          length = textbox.value.length;
        }
        textbox.setSelectionRange(0, length);
      }, 1);
    },

    setLayoutSettingChanged: function(flag) {
      this.layoutSettingChanged = flag;
    },

    isLayoutSettingChanged: function() {
      return this.layoutSettingChanged;
    },

    confirmDiscardLayoutSetting: function() {
      if (wagbyDesigner.isLayoutSettingChanged() &&
          !confirm(this.jfcRes.MSG_CONFIRM_LAYOUT_SETTING_UNSAVED)) {
        return false;
      }
      this.setLayoutSettingChanged(false);
      return true;
    },

    openWagbyManualPage: function(page) {
      var url = "http://wagby.com/";
      if (this.locale && this.locale.toLowerCase().lastIndexOf("ja", 0) === -1) {
        url = url + "en/";
      }
      url = url + "manual8/";
      if (page) {
        url = url + page;
      }
      this.MANUAL_WINDOW = window.open(url, "MANUAL_WINDOW");
    },

    createEventSource: function(args) {
      var eventSource = new EventSource(args.url);
      if (!eventSource) {
        return null;
      }
      if (args.onOpen) {
        eventSource.addEventListener("open", function (event) {
          args.onOpen(event);
        });
      }
      eventSource.addEventListener("message", function (event) {
        if (args.onMessage) {
          args.onMessage(event, eventSource);
        }
        if (event.data === "EventSourceComplete") {
          if (args.onComplete) {
            args.onComplete(event);
          }
          eventSource.close();
        }
      });
      eventSource.addEventListener("error", function (event) {
        eventSource.close();
        if (args.onError) {
          args.onError(event);
        }
      });
      return eventSource;
    },

    setBrowserType: function() {
      this.isChrome = has("chrome");
      this.isFirefox = has("ff");
      this.isEdge = has("edge");
    },

    isMessageQueueEnabled: function() {
      return this.projectEnvironmentPane.isMessageQueueEnabled();
    },

    getDisplayitemSortFunc: function() {
      var reverseSortFlag = 1;
      if (!(this.isEdge || this.isChrome && this.isChrome >= 70)) {
        // ブラウザによってソート順が異なるため、Chrome 69 以前、
        // Edge 以外のブラウザの場合はソートを逆順にする
        reverseSortFlag = -1;
      }
      return function(a, b) {
        if (!a.order || !b.order ||
            (a.order === wagbyDesigner.jfcRes.OPTION_CHECKED && b.order === wagbyDesigner.jfcRes.OPTION_CHECKED) ||
            (a.order !== wagbyDesigner.jfcRes.OPTION_CHECKED && b.order === wagbyDesigner.jfcRes.OPTION_CHECKED)) {
          return 1 * reverseSortFlag;
        }
        if (a.order === wagbyDesigner.jfcRes.OPTION_CHECKED &&
            b.order !== wagbyDesigner.jfcRes.OPTION_CHECKED) {
          return -1 * reverseSortFlag;
        }
        try {
          var aOrder = parseInt(a.order, 10);
          var bOrder = parseInt(b.order, 10);
          return aOrder - bOrder;
        } catch(e) {
          console.error(e);
        }
        return 1 * reverseSortFlag;
      }
    },

    setUserModelListData: function() {
      var that = this;
      this.project.getUserModelListData(function(data) {
        that.modelListPane.userModelListPaneContent.createModelList(data);
        that.paneManager.show(wagbyDesigner.modelListPane);
        that.modelListPane.tabContainer.layout();
        that.fadeOutLoader();
      });
    },

    clearModelListCache: function() {
      this.modelListPane.userModelListPaneContent.clearCache();
      this.modelListPane.systemModelListPaneContent.clearCache();
    },

    releaseLock: function(callback, argSync) {
      var sync = argSync || false;
      this.sendRequest({
        'url': "/lock/release/thissession",
        'options': {
          'handleAs': 'json',
          'method': "POST",
          'sync': sync,
        },
      }, callback);
    },

    releaseLockSync: function(callback) {
      this.releaseLock(callback, true);
    },

    getOpenFontAwesomeWindowFunc: function() {
      var that = this;
      return function() {
        that.openFontAwesomeWindow();
      };
    },

    openFontAwesomeWindow: function() {
      if (this.FA_WINDOW && !this.FA_WINDOW.closed) {
        this.FA_WINDOW.focus();
        return;
      }
      this.FA_WINDOW = window.open(this.FA_URL, "FA_WINDOW");
    },

    setMarkSelectOption: function(selectWidget, selectedOptionValues) {
      var options = selectWidget.get("options");
      for (var i=0,l=options.length; i<l; i++) {
        var option = options[i];
        if (selectedOptionValues[option.value]) {
          this.setMarkToOptionLabel(selectWidget, option);
        } else {
          this.removeMarkFromOptionLabel(selectWidget, option);
        }
      }
    },

    setMarkToOptionLabel: function(selectWidget, option) {
      if (this.isMarkSetOptionLabel(option)) {
        return;
      }
      option.label = this.SCRIPT_DEFINED_MARK + option.label;
      selectWidget.updateOption(option);
    },

    removeMarkFromOptionLabel: function(selectWidget, option) {
      if (!this.isMarkSetOptionLabel(option)) {
        return;
      }
      option.label = option.label.substring(this.SCRIPT_DEFINED_MARK.length);
      selectWidget.updateOption(option);
    },

    isMarkSetOptionLabel: function(option) {
      if (!option || !option.label) {
        return false;
      }
      if (option.label.lastIndexOf(this.SCRIPT_DEFINED_MARK, 0) === 0) {
        return true;
      }
      return false;
    },

    getModelDescription: function(model, modelId) {
      if (!model) {
        return modelId;
      }
      return model.getDescriptionWithId();
    },

    getModelitemLabelWithName: function(model, modelitemName) {
      var modelitem = model.getModelitemByName(modelitemName);
      if (!modelitem) {
        return modelitemName;
      }
      return modelitem.getLabelWithName();
    },

  });
});
